#include <extdll.h>			// always
#include <meta_api.h>		// of course
#include <dllapi.h>
#include "sdk_util.h"		// UTIL_LogPrintf, etc
#include <h_export.h>
#include "plugin.h"
#include "cbase.h"
#include <usercmd.h>
#include <pm_defs.h>

enginefuncs_t g_engfuncs;
globalvars_t  *gpGlobals;
locationinfo_t LocationInfo[128];
playerdata_t PlayerData[33];

int CurrentMessage;
int CurrentArg;
int NumLocations = 0;
bool DidMsg = false;
float NextEntityCheck = 0;

DLL_GLOBAL const Vector g_vecZero = Vector(0,0,0);

void CmdStart( const edict_t *player, const struct usercmd_s *_cmd, unsigned int random_seed )
{
	edict_t *pEntity = (edict_t *)player;
	struct usercmd_s * cmd     = (struct usercmd_s *)_cmd;
	
	if(ENTINDEX(pEntity) < 1 || ENTINDEX(pEntity) > 32)
	{
		RETURN_META(MRES_IGNORED);
	}

	if(cmd->impulse == 55)
	{
		if(UTIL_CountBuildings("phasegate") > 8)//We limit phase gates to 8 to prevent problems.
		{
			if (gmsgTextMsg == 0)
				gmsgTextMsg = REG_USER_MSG( "TextMsg", -1 );
			
			MESSAGE_BEGIN( MSG_ONE, gmsgTextMsg, NULL, pEntity );
			WRITE_BYTE( HUD_PRINTTALK );
			WRITE_STRING( "* Max Phase Gates Reached (8 Phase Gates)\n");
			MESSAGE_END();

			cmd->impulse = 0;
			RETURN_META(MRES_SUPERCEDE);
		}
	}

	if(PlayerData[ENTINDEX(pEntity)].iInPhaseMenu)
	{
		if(cmd->buttons & IN_FORWARD 
			|| cmd->buttons & IN_BACK
			|| cmd->buttons & IN_MOVELEFT
			|| cmd->buttons & IN_MOVERIGHT)
		{
			PlayerData[ENTINDEX(pEntity)].iInPhaseMenu = 0;
			PlayerData[ENTINDEX(pEntity)].iLastPhaseUsed = 0;
			UTIL_ShowMenu(pEntity, 1, 1, FALSE, "\n");
		}
	}
	RETURN_META(MRES_HANDLED);
}

void StartFrame( void )
{
	if(NextEntityCheck < gpGlobals->time)
	{
		CBaseEntity *pCPhaseGate = NULL;
		while(pCPhaseGate = UTIL_FindEntityByClassname(pCPhaseGate, "phasegate"))//Phasegate doesnt normally have nextthink, so we have to put one on.
		{
			if(!pCPhaseGate->pev->iuser1)//Only put it on once.
			{
				pCPhaseGate->pev->nextthink = gpGlobals->time + 0.1;
				pCPhaseGate->pev->iuser1 = 1;
			}
		}
		NextEntityCheck = gpGlobals->time + 0.25;
	}
	RETURN_META(MRES_HANDLED);
}

int DispatchSpawn(edict_t *pEntity)
{
	if(FStrEq(STRING(pEntity->v.classname),"worldspawn"))
	{
		memset(PlayerData,0,sizeof(playerdata_t)*33);
	}
	RETURN_META_VALUE(MRES_HANDLED,0);
}

void DispatchThink(edict_t *pent)
{
	if(!pent)
		RETURN_META(MRES_IGNORED);

	if(FStrEq(STRING(pent->v.classname), "phasegate"))
	{
		if(UTIL_IsBuilt(pent))
		{
			CBaseEntity *pEnt = CBaseEntity::Instance(pent);
			if(pEnt->m_pfnUse != &CBaseEntity::m_pfnPhaseUse)
			{
				pEnt->m_pfnUse =  &CBaseEntity::m_pfnPhaseUse;
			}
		}
		pent->v.nextthink = gpGlobals->time + 0.25;
	}
	RETURN_META(MRES_HANDLED);
}

BOOL ClientConnect(edict_t *pEntity, const char *pszName, const char *pszAddress, char szRejectReason[ 128 ] )
{
	memset(&PlayerData[ENTINDEX(pEntity)],0,sizeof(playerdata_t));
	RETURN_META_VALUE(MRES_HANDLED,TRUE);
}

void PlayerPreThink(edict_t *pEntity)
{
	if(PlayerData[ENTINDEX(pEntity)].iInPhaseMenu && PlayerData[ENTINDEX(pEntity)].iLastPhaseUsed)
	{
		edict_t *pPhase = INDEXENT(PlayerData[ENTINDEX(pEntity)].iLastPhaseUsed);
		if(FNullEnt(pPhase))
		{
			PlayerData[ENTINDEX(pEntity)].iInPhaseMenu = 0;
			UTIL_ShowMenu(pEntity, 1, 1, FALSE, "\n");
		}
	}
	RETURN_META(MRES_HANDLED);
}




void ClientCommand(edict_t *pEntity)
{
	if(PlayerData[ENTINDEX(pEntity)].iInPhaseMenu)
	{
		if(FStrEq(CMD_ARGV(0), "menuselect"))
		{
			int Option = atoi(CMD_ARGV(1));
			
			switch(Option)
			{
			case 9:
				PlayerData[ENTINDEX(pEntity)].iNumPhases = 0;
				PlayerData[ENTINDEX(pEntity)].iInPhaseMenu = 0;
				memset(PlayerData[ENTINDEX(pEntity)].iPhaseGates, 0, sizeof(PlayerData[ENTINDEX(pEntity)].iPhaseGates));
				RETURN_META(MRES_SUPERCEDE);
				break;
			default:
				if(Option > PlayerData[ENTINDEX(pEntity)].iNumPhases + 1)
					break;
				
				edict_t *pPhase = INDEXENT(PlayerData[ENTINDEX(pEntity)].iPhaseGates[Option]);
				
				if(UTIL_IsValidEntity(pPhase) && FStrEq(STRING(pPhase->v.classname),"phasegate"))
				{
					PLAYBACK_EVENT_FULL( 0, pEntity, 39, 0, (float *)(&(pEntity->v.origin)), (float *)&g_vecZero, 0.0, 0.0, 0, 0, 0, 0 );
					EMIT_SOUND(pEntity, CHAN_ITEM, "misc/transport2.wav", 1, ATTN_NORM);
					
					pEntity->v.velocity = Vector(0,0,0);
					pEntity->v.origin = pPhase->v.origin + Vector(0,0,80);
					
					UTIL_MakeVectors(pEntity->v.angles);
					pEntity->v.velocity = pEntity->v.velocity + gpGlobals->v_forward * 300;
					
					PLAYBACK_EVENT_FULL( 0, pEntity, 39, 0, (float *)(&(pEntity->v.origin)), (float *)&g_vecZero, 0.0, 0.0, 0, 0, 0, 0 );
					EMIT_SOUND(pEntity, CHAN_ITEM, "misc/transport2.wav", 1, ATTN_NORM);
					
					
					CBaseEntity *pCEntity = NULL;
					pCEntity = CBaseEntity::Instance(pEntity);
					if(pCEntity != NULL)
					{
						pCEntity->ScreenFade(Vector( 255, 255, 255), 100, 1, 0.25, 0);
					}
					PlayerData[ENTINDEX(pEntity)].iInPhaseMenu = 0;
					PlayerData[ENTINDEX(pEntity)].flPhaseDelay = gpGlobals->time + 2.5;
					PlayerData[ENTINDEX(pEntity)].iLastPhaseUsed = 0;
				}
				PlayerData[ENTINDEX(pEntity)].iInPhaseMenu = 0;
				PlayerData[ENTINDEX(pEntity)].iLastPhaseUsed = 0;
				PlayerData[ENTINDEX(pEntity)].iNumPhases = 0;
				memset(PlayerData[ENTINDEX(pEntity)].iPhaseGates, 0, sizeof(PlayerData[ENTINDEX(pEntity)].iPhaseGates));
				RETURN_META(MRES_SUPERCEDE);
			}
			RETURN_META(MRES_SUPERCEDE);
		}
	}
	RETURN_META(MRES_HANDLED);
}

void MessageBegin(int msg_dest, int msg_type, const float *pOrigin, edict_t *ed)
{
	if(!DidMsg)//This is the only place I could figure out where to put this.
	{
		SAFE_USER_MSG(gmsgSetupMap,   "SetupMap", 1);
		SAFE_USER_MSG(gmsgTextMsg,	  "TextMsg", -1);
		SAFE_USER_MSG(gmsgScreenFade, "ScreenFade", 12);
		SAFE_USER_MSG(gmsgShowMenu,   "ShowMenu", -1);
		DidMsg = true;
	}
	CurrentMessage = msg_type;
	CurrentArg = 1;
	RETURN_META(MRES_HANDLED);
}

void MessageEnd( void )
{
	CurrentMessage = 0;
	CurrentArg = 0;
	RETURN_META(MRES_HANDLED);
}

void WriteString(const char *sz)
{
	if(CurrentMessage == gmsgSetupMap)
	{
		sprintf(LocationInfo[NumLocations].Name, "%s", LookupLocationTitle(sz));//Look up proper title from titles.txt
	}

	CurrentArg++;
	RETURN_META(MRES_HANDLED);
}

void WriteByte(int iValue)
{
	if(CurrentMessage == gmsgSetupMap)
	{
		if(iValue == 0 && CurrentArg == 1)
		{
			NumLocations = 1;
		}
		else if (iValue == 1 && CurrentArg == 1)
		{
			NumLocations++;
		}
	}
	CurrentArg++;
	RETURN_META(MRES_HANDLED);
}


void WriteCoord(float flValue)
{
	if(CurrentMessage == gmsgSetupMap)  //We need to catch all of the Location Origins, and store them into an array
	{
		if(CurrentArg == 3)
			LocationInfo[NumLocations].Origin[0].x = flValue;
		else if(CurrentArg == 4)
			LocationInfo[NumLocations].Origin[0].y = flValue;
		else if(CurrentArg == 5)
			LocationInfo[NumLocations].Origin[1].x = flValue;
		else if(CurrentArg == 6)
			LocationInfo[NumLocations].Origin[1].y = flValue;
	}
	CurrentArg++;
	RETURN_META(MRES_HANDLED);
}


// Must provide at least one of these..
static META_FUNCTIONS gMetaFunctionTable = {
		NULL,			// pfnGetEntityAPI				HL SDK; called before game DLL
		NULL,			// pfnGetEntityAPI_Post			META; called after game DLL
		GetEntityAPI2,	// pfnGetEntityAPI2				HL SDK2; called before game DLL
		NULL,			// pfnGetEntityAPI2_Post		META; called after game DLL
		NULL,			// pfnGetNewDLLFunctions		HL SDK2; called before game DLL
		NULL,			// pfnGetNewDLLFunctions_Post	META; called after game DLL
		GetEngineFunctions,			// pfnGetEngineFunctions		META; called before HL engine
		NULL,			// pfnGetEngineFunctions_Post	META; called after HL engine
};

// Description of plugin
plugin_info_t Plugin_info = {
	META_INTERFACE_VERSION,	// ifvers
		"PhaseSelect",	// name
		"1.0",		// version
		"5/9/2003",	// date
		"voogru <voogru@voogru.com>",	// author
		"http://www.voogru.com",	// url
		"PHASESELECT",	// logtag
		PT_CHANGELEVEL,// (when) loadable
		PT_CHANGELEVEL,// (when) unloadable
};

// Global vars from metamod:
meta_globals_t *gpMetaGlobals;		// metamod globals
gamedll_funcs_t *gpGamedllFuncs;	// gameDLL function tables
mutil_funcs_t *gpMetaUtilFuncs;		// metamod utility functions

C_DLLEXPORT int Meta_Query(char *ifvers, plugin_info_t **pPlugInfo,	mutil_funcs_t *pMetaUtilFuncs) {
	*pPlugInfo=&Plugin_info;
	gpMetaUtilFuncs=pMetaUtilFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Attach(PLUG_LOADTIME now, META_FUNCTIONS *pFunctionTable, meta_globals_t *pMGlobals, gamedll_funcs_t *pGamedllFuncs) {
	gpMetaGlobals=pMGlobals;
	memcpy(pFunctionTable, &gMetaFunctionTable, sizeof(META_FUNCTIONS));
	gpGamedllFuncs=pGamedllFuncs;
	return(TRUE);
}

C_DLLEXPORT int Meta_Detach(PLUG_LOADTIME now, PL_UNLOAD_REASON reason) 
{
	return(TRUE);
}

void WINAPI GiveFnptrsToDll( enginefuncs_t* pengfuncsFromEngine, globalvars_t *pGlobals ) {
	memcpy(&g_engfuncs, pengfuncsFromEngine, sizeof(enginefuncs_t));
	gpGlobals = pGlobals;		
}

DLL_FUNCTIONS gFunctionTable;
C_DLLEXPORT int GetEntityAPI2( DLL_FUNCTIONS *pFunctionTable, int *interfaceVersion ) 
{
	gFunctionTable.pfnCmdStart = CmdStart;
	gFunctionTable.pfnSpawn = DispatchSpawn;
	gFunctionTable.pfnThink = DispatchThink;
	gFunctionTable.pfnClientConnect = ClientConnect;
	gFunctionTable.pfnClientCommand = ClientCommand;
	gFunctionTable.pfnPlayerPreThink = PlayerPreThink;
	gFunctionTable.pfnStartFrame = StartFrame;
	memcpy( pFunctionTable, &gFunctionTable, sizeof( DLL_FUNCTIONS ) );
	return(TRUE);
}



enginefuncs_t meta_engfuncs;
C_DLLEXPORT int GetEngineFunctions(enginefuncs_t *pengfuncsFromEngine, int *interfaceVersion ) 
{
	meta_engfuncs.pfnMessageBegin = MessageBegin;
	meta_engfuncs.pfnMessageEnd = MessageEnd;
	meta_engfuncs.pfnWriteByte = WriteByte;
	meta_engfuncs.pfnWriteString = WriteString;
	meta_engfuncs.pfnWriteCoord = WriteCoord;
	memcpy(pengfuncsFromEngine, &meta_engfuncs, sizeof(enginefuncs_t));
	return(TRUE);
}


